@@ -974,11 +974,6 @@ static PyObject *
 _wrap_pyg_type_register(PyObject *self, PyObject *args)
 {
     PyTypeObject *class;
-    PyObject *gtype;
-
-    if (PyErr_Warn(PyExc_DeprecationWarning, "gobject.type_register is deprecated;"
-                   " now types are automatically registered"))
-        return NULL;
 
     if (!PyArg_ParseTuple(args, "O:gobject.type_register", &class))
 	return NULL;
@@ -988,16 +983,25 @@ _wrap_pyg_type_register(PyObject *self, PyObject *args)
     }
 
       /* Check if type already registered */
-    gtype = PyObject_GetAttrString((PyObject *)class, "__gtype__");
-    if (gtype == NULL) {
-        PyErr_Clear();
-          /* not registered */
+    if (pyg_type_from_object((PyObject *) class) ==
+        pyg_type_from_object((PyObject *) class->tp_base))
+    {
         if (pyg_type_register(class, NULL))
             return NULL;
-    } else
+    } else {
+        gchar *msg;
           /* already registered */
-        Py_DECREF(gtype);
-
+        msg = g_strdup_printf("Class %s is already GObject-registered; "
+			      "Please note that classes containing any of the "
+			      "attributes __gtype_name__, __gproperties__, or "
+			      "__gsignals__ are now automatically registered.",
+			      class->tp_name);
+        if (PyErr_Warn(PyExc_DeprecationWarning, msg)) {
+            g_free(msg);
+            return NULL;
+        }
+        g_free(msg);
+    }
     Py_INCREF(class);
     return (PyObject *) class;
 }
@@ -1095,7 +1099,7 @@ pygobject__g_instance_init(GTypeInstance   *instance,
 }
 
 int
-pyg_type_register(PyTypeObject *class, char *type_name)
+pyg_type_register(PyTypeObject *class, const char *type_name)
 {
     PyObject *gtype, *gsignals, *gproperties, *overridden_signals;
     GType parent_type, instance_type;
@@ -1117,7 +1121,7 @@ pyg_type_register(PyTypeObject *class, char *type_name)
 	0,    /* n_preallocs */
 	(GInstanceInitFunc) pygobject__g_instance_init
     };
-
+    gchar *new_type_name;
 
     /* find the GType of the parent */
     parent_type = pyg_type_from_object((PyObject *)class);
@@ -1125,8 +1129,10 @@ pyg_type_register(PyTypeObject *class, char *type_name)
 	return -1;
     }
 
-    if (!type_name)
-	type_name = get_type_name_for_class(class);
+    if (type_name)
+        new_type_name = (gchar *) type_name; /* care is taken below not to free this */
+    else
+	new_type_name = get_type_name_for_class(class);
     
     /* set class_data that will be passed to the class_init function. */
     type_info.class_data = class;
@@ -1137,9 +1143,10 @@ pyg_type_register(PyTypeObject *class, char *type_name)
     type_info.instance_size = query.instance_size;
 
     /* create new typecode */
-    instance_type = g_type_register_static(parent_type, type_name,
+    instance_type = g_type_register_static(parent_type, new_type_name,
 					   &type_info, 0);
-    g_free(type_name);
+    if (type_name == NULL)
+        g_free(new_type_name);
     if (instance_type == 0) {
 	PyErr_SetString(PyExc_RuntimeError, "could not create new GType");
 	return -1;
@@ -1158,10 +1165,7 @@ pyg_type_register(PyTypeObject *class, char *type_name)
       /* propagate new constructor API compatility flag from parent to child type */
     has_new_constructor_api = g_type_get_qdata(parent_type,
                                                pygobject_has_updated_constructor_key);
-    if (has_new_constructor_api == NULL)
-        g_warning("Constructor wrapper for %s needs to be updated to the new API",
-                  g_type_name(parent_type));
-    else
+    if (has_new_constructor_api != NULL)
         g_type_set_qdata(instance_type, pygobject_has_updated_constructor_key,
                          has_new_constructor_api);
 
@@ -1311,12 +1315,12 @@ pyg_signal_list_names (PyObject *self, PyObject *args, PyObject *kwargs)
 {
     static char *kwlist[] = { "type", NULL };
     PyObject *py_itype, *list;
-    GObjectClass *class;
+    GObjectClass *class = NULL;
     GType itype;
     guint n;
     guint *ids;
     guint i;
-
+    
     if (!PyArg_ParseTupleAndKeywords(args, kwargs,
                                      "O:gobject.signal_list_names",
                                      kwlist, &py_itype))
@@ -1324,31 +1328,31 @@ pyg_signal_list_names (PyObject *self, PyObject *args, PyObject *kwargs)
     if ((itype = pyg_type_from_object(py_itype)) == 0)
 	return NULL;
 
-    if (!G_TYPE_IS_INSTANTIATABLE(itype) && !G_TYPE_IS_INTERFACE(itype)) {
+    if (G_TYPE_IS_INSTANTIATABLE(itype)) {
+	class = g_type_class_ref(itype);
+	if (!class) {
+	    PyErr_SetString(PyExc_RuntimeError,
+			    "could not get a reference to type class");
+	    return NULL;
+	}
+    } else if (!G_TYPE_IS_INTERFACE(itype)) {
 	PyErr_SetString(PyExc_TypeError,
 			"type must be instantiable or an interface");
 	return NULL;
     }
-
-    class = g_type_class_ref(itype);
-    if (!class) {
-	PyErr_SetString(PyExc_RuntimeError,
-			"could not get a reference to type class");
-	return NULL;
-    }
+    
     ids = g_signal_list_ids(itype, &n);
 
     list = PyTuple_New((gint)n);
-    if (list == NULL) {
-	g_free(ids);
-	g_type_class_unref(class);
-	return NULL;
+    if (list != NULL) {
+	for (i = 0; i < n; i++)
+	    PyTuple_SetItem(list, i,
+			    PyString_FromString(g_signal_name(ids[i])));
     }
-
-    for (i = 0; i < n; i++)
-	PyTuple_SetItem(list, i, PyString_FromString(g_signal_name(ids[i])));
+    
     g_free(ids);
-    g_type_class_unref(class);
+    if (class)
+	g_type_class_unref(class);
     return list;
 }
 
@@ -1649,10 +1653,11 @@ pyg_object_new (PyGObject *self, PyObject *args, PyObject *kwargs)
     g_type_class_unref(class);
     
     if (obj)
-	self = (PyGObject *) pygobject_new ((GObject *)obj);
+	self = (PyGObject *) pygobject_new_full((GObject *)obj, FALSE);
     else
         self = NULL;
     g_object_unref(obj);
+    pygobject_sink(obj);
 
     return (PyObject *) self;
 }
@@ -1894,18 +1899,9 @@ pyg_source_remove(PyObject *self, PyObject *args)
 }
 
 static PyObject *
-pyg_main_context_default (PyObject *unused)
+pyg_main_context_default(PyObject *unused)
 {
-    PyGMainContext *self;
-
-    self = (PyGMainContext *)PyObject_NEW(PyGMainContext,
-					  &PyGMainContext_Type);
-    if (self == NULL)
-	return NULL;
-
-    self->context = g_main_context_default();
-    return (PyObject *)self;
-
+    return pyg_main_context_new(g_main_context_default());
 }
 
 static int pyg_thread_state_tls_key = -1;
@@ -1913,7 +1909,7 @@ static int pyg_thread_state_tls_key = -1;
 /* Enable threading; note that the GIL must be held by the current
    thread when this function is called */
 static int
-pyg_enable_threads ()
+pyg_enable_threads (void)
 {
     if (getenv ("PYGTK_USE_GIL_STATE_API"))
 	use_gil_state_api = TRUE;
@@ -2716,13 +2712,15 @@ initgobject(void)
 
     REGISTER_TYPE(d, PyGTypeWrapper_Type, "GType");
 
-    PY_TYPE_OBJECT = g_boxed_type_register_static("PyObject",
-						  pyobject_copy,
-						  pyobject_free);
+    if (!PY_TYPE_OBJECT)
+	PY_TYPE_OBJECT = g_boxed_type_register_static("PyObject",
+						      pyobject_copy,
+						      pyobject_free);
 
     gerror_exc = PyErr_NewException("gobject.GError", PyExc_RuntimeError,NULL);
     PyDict_SetItemString(d, "GError", gerror_exc);
 
+    PyGObject_MetaType.tp_base = &PyType_Type;
     PyGObject_MetaType.tp_traverse = PyType_Type.tp_traverse;
     PyGObject_MetaType.tp_clear = PyType_Type.tp_clear;
     PyGObject_MetaType.tp_is_gc = PyType_Type.tp_is_gc;
